<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearStory;
use App\Model\Admin\HomegearRoom;
use App\Model\Admin\HomegearBuilding;

class StoryController extends Controller
{
    public function indexAction(Request $request, HomegearStory $story, HomegearBuilding $buildings)
    {
        return $this->render('Admin/Inventory/Story/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'stories' => $story->loadAll(true),
            'buildings' => $buildings->loadAll()
        ));
    }

    public function editAction($id, Request $request, HomegearStory $story, HomegearRoom $room)
    {
        // get the current story
        $currentStory = $story->loadById($id);
        // get all known stories & rooms
        $allStories = $story->loadAll();
        $rooms = $room->loadAll();
        // Check every room if it's associated to a story. This must only be possible once!
        foreach ($allStories as $myStory) {
            if (array_key_exists('ROOMS',$myStory) && $myStory['ROOMS'] !== NULL) {
                foreach($myStory['ROOMS'] as $k => $v) {
                    // Current story is using array with the sorting parameter. So check if this is the current story or not
                    if (is_array($v) && array_key_exists('id',$v)) {
                        $key = array_search($v['id'], array_column($rooms, 'ID'));
                    } else {
                        $key = array_search($v, array_column($rooms, 'ID'));
                    }
                    // Check if a room was found within the story
                    if (isset($key) && $key !== NULL) {
                        // It is the current story, so store room-data
                        if ($myStory['ID'] == $currentStory['ID']) {
                            $sort = $currentStory['ROOMS'][$k]['sort'];
                            $currentStory['ROOMS'][$k] = ['id' => $rooms[$key]['ID'], 'name' => $rooms[$key]['TRANSLATIONS'], 'sort' => $sort];
                        }
                        // Flag room to be found within a story, so it's not visible for others anymore
                        $rooms[$key]['inStory'] = true;
                    }
                }
            }
        }
        return $this->render('Admin/Inventory/Story/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'story' => $currentStory,
            'allRooms' => $rooms
        ));
    }

    public function updateAction($id, Request $request, HomegearStory $story)
    {
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'name' => new Assert\NotBlank,
            'goback' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $story->update($id,$params['name']);
        }
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.stories.edit', array('id' => $id));
        }
    }

    public function createAction(Request $request, HomegearStory $story, HomegearBuilding $building)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'name' => new Assert\NotBlank,
                'lang' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $story->create($params['name'],$params['lang']);
            if (array_key_exists('building',$params) && $params['building'] != '') {
                $building->addStory($params['building'],[$result]);
            }
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.stories');
    }

    public function deleteAction($id, Request $request, HomegearStory $story)
    {
        $story->delete($id);
        return $this->redirectToRoute('admin.inventory.stories');
    }

    public function updateSortAction($id, Request $request, HomegearStory $story)
    {
        $params = $request->request->all();
        $story->updateSort($id,$params['position']);
        return new JsonResponse(array('success' => 1));
    }

    public function addRoomAction($id, Request $request, HomegearStory $story)
    {
        $params = $request->request->all();
        $story->addRoom($id,$params['room']);
        return new JsonResponse(array('success' => 1));
    }

    public function removeRoomAction($id, Request $request, HomegearStory $story)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'roomid' => new Assert\NotBlank
        ));
        $params = $request->query->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $story->removeRoom($id,$params['roomid']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.stories.edit', array('id' => $id));
    }

    public function updateRoomSortAction($id, Request $request, HomegearStory $story)
    {
        $params = $request->request->all();
        $story->updateRoomSort($id,$params['roomId'],$params['position']);
        return new JsonResponse(array('success' => 1));
    }

}