<?php
Namespace App\Controller\Admin\Maintenance;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use App\Model\Admin\HomegearService;
use App\Utils\Utils;

class ServiceController extends Controller
{
    public function indexAction(Request $request, HomegearService $services, Utils $utils)
    {
        $showPwdChange = true;
        if (substr($_SERVER['REQUEST_URI'],0,6) === '/admin') {
            if (! $utils->checkUserExists('user')) { $showPwdChange = false; }
            $serviceList = ['homegear-influxdb', 'influxdb', 'mosquitto', 'ssh'];
        } else {
            $serviceList = ['homegear-influxdb', 'influxdb', 'mosquitto', 'openvpn@cloudmatic', 'ssh'];
        }
        return $this->render('Admin/Maintenance/Services/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'mgmt' => $utils->checkManagementRunning(),
            'currentEnv' => (array_key_exists('APP_ENV',$_SERVER) ? $_SERVER['APP_ENV'] : 'prod'),
            'currentDebug' => (array_key_exists('APP_DEBUG',$_SERVER) ? $_SERVER['APP_DEBUG'] : 0),
            'services' => $services->list($serviceList),
            'showPwdChange' => $showPwdChange,
            'oled' => $services->checkOled()
        ));
    }

    public function startAction($service, Request $request, HomegearService $services)
    {
        $result = $services->start($service);
        return new JsonResponse($result);
    }

    public function stopAction($service, Request $request, HomegearService $services)
    {
        $result = $services->stop($service);
        return new JsonResponse($result);
    }

    public function restartAction($service, Request $request, HomegearService $services)
    {
        $result = $services->restart($service);
        return new JsonResponse($result);
    }

    public function rebootAction(Request $request, HomegearService $services)
    {
        $services->reboot();
        return new JsonResponse();
    }

    public function shutdownAction(Request $request, HomegearService $services)
    {
        $result = $services->shutdown();
        return new JsonResponse($result);
    }

    public function setSshPasswordAction(Request $request, HomegearService $services)
    {
        $params = $request->request->all();
        $result = $services->setSshPassword($params);
        return new JsonResponse($result);
    }

    public function setEnvAction(Request $request, HomegearService $services)
    {
        $params = $request->request->all();
        $result = $services->setEnv($params['env']);
        return new JsonResponse($result);
    }

    public function setOledAction(Request $request, HomegearService $services)
    {
        $params = $request->request->all();
        $result = $services->setOled($params['oled_status']);
        return $this->redirectToRoute('admin.maintenance.services',array(),301);
    }
}
