<?php
Namespace App\Model\Admin;

Use App\Utils\Utils;

/**
 * Provides admin functions for CloudMatic management
 *
 */
class CloudMatic
{
    public $hg;
    private $utils;
    private $baseUrl = 'https://www.cloudmatic.de/';
    private $mailCheck = 'https://www.cloudmatic.de/script/checkMailAddress.php';
    private $registerUrl = 'https://www.cloudmatic.de/userapi.php';
    private $key = "EZ37wshg37DH228shSHq";
    private $keyscript = 'getkeyccu2.php';
    private $workDir = '/tmp/cloudmatic';
    private $allowed =  array('gz', 'ovpn', 'conf');
    private $ocservUserdb = false;

    public function __construct(Utils $utils) {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->ocservUserdb = '/etc/ocserv/user.passwd';
    }

    /**
     * Load CloudMatic Settings, if any
     * 
     * @return array()
     */
    public function load() {
        // check if there is a CloudMatic Account
        $cm = $this->hg->getData('CloudMatic');
        if (is_array($cm) && isset($cm['userid'])) {
            // get subscription data
            $data = $this->_getSubscriptionData($cm['userid'],$cm['userkey']);
            if (is_array($data)) {
                $cm = array_merge($cm,$data);
            }
            // check if OpenVPN is running
            if ($this->utils->checkManagementRunning(false)) {
                $id = $this->hg->managementServiceCommand('openvpn@cloudmatic', 'status');
                $result = $this->utils->getCommandStatus($id);
                $result = explode(PHP_EOL,$result['output']);
                if (stristr($result[1],'loaded') && stristr($result[1],'enabled')) { $cm['openvpn_enabled'] = true; }
                $status = explode(': ',$result[2]);
                $cm['openvpn_status'] = trim($status[1]);
            } else {
                $cm['openvpn_status'] = "n/a";
                $cm['openvpn_enabled'] = "n/a";
            }
            // check for tunnel
            $tunnel = $this->utils->getIpAddresses('tun0');
            if (is_array($tunnel) && array_key_exists('name',$tunnel) && $tunnel['name'] == 'tun0') {
                // if result is an array, tunnel is offline (eg. all interfaces are returned!)
                $cm['tunnel'] = true;
            } else {
                $cm['tunnel'] = false;
            }
            // check cluster
            exec("ping -c 1 -W 1 37.187.191.205", $output, $result);
            $cm['cluster_external'] = ((is_array($output) && array_key_exists(1,$output) && strpos($output[1],'icmp_seq')) !== false ? true : false);
            $ptp = $this->_getPTPAddress();
            if (isset($ptp) && $ptp != '') {
                exec("ping -c 1 -W 1 $ptp", $output1, $result);
                $cm['cluster_internal'] = ((is_array($output1) && array_key_exists(1,$output1) && strpos($output1[1],'icmp_seq')) !== false ? true : false);
            } else {
                $cm['cluster_internal'] = false;
            }
        }
        return $cm;
    }

    /**
     * Check if OpenVPN is installed
     * 
     * @return bool
     */
    public function checkOpenVPN() {
        if (!file_exists('/usr/sbin/openvpn')) { return false; }
        return true;
    }

    /**
     * Restart OpenVPN
     * 
     * @return bool or result
     */
    public function restartOpenVPN() {
        $id = $this->hg->managementServiceCommand('openvpn@cloudmatic','restart');
        $result = $this->utils->getCommandStatus($id);
        if ($result['exitCode'] == 0) {
            return 'success';
        }
        return $result['output'];
    }

    /**
     * Do a "manual" key update
     * 
     * @return
     */
    public function manualUpdate() {
        $cmid = $this->hg->getData('CloudMatic','userid');
        $cmkey = $this->hg->getData('CloudMatic','userkey');
        if ($cmid == '' || $cmkey == '') { return 'Fatal error: CloudMatic ID and Key not found!'; }
        if (!file_exists($this->workDir)) {
            mkdir($this->workDir, 0700);
        }
        $this->_keyDownload($this->baseUrl.$this->keyscript."?id=$cmid&key=$cmkey");
        exec(escapeshellcmd('tar -xzvf '.$this->workDir.'/key.tar.gz -C '.$this->workDir),$result);
        // check if key is newer
        $version = file($this->workDir.'/newver');
        if (trim($version[0]) > $this->hg->getData('CloudMatic','Version')) {
            // version is newer, install new files
            $result = $this->_installFiles();
            if ($result == 1) {
                // all fine, set new version & restart service
                $this->hg->setData('CloudMatic','Version',trim($version[0]));
                $id = $this->hg->managementServiceCommand("openvpn@cloudmatic", 'restart');
                $result = $this->utils->getCommandStatus($id);
                if ($result['exitCode'] == 0) {
                    exec(escapeshellcmd('rm -rf '.$this->workDir));
                    return 'success';
                }
                exec(escapeshellcmd('rm -rf '.$this->workDir));
                return 'Update installed, but OpenVPN could not be restarted:<br>'.$result[1];
            }
            exec(escapeshellcmd('rm -rf '.$this->workDir));
            return 'Update could not be installed. Please try again later.';
        }
        exec(escapeshellcmd('rm -rf '.$this->workDir));
        return 'No new Update available';
    }

    /**
     * Handle upload keyfile
     * 
     * @return
     */
    public function uploadKeyfile($file) {
        $filename = $file->getClientOriginalName();
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        if(!in_array($extension,$this->allowed)) { return 'File extension not allowed.'; }
        // these are plain configs
        if ($extension == "ovpn" || $extension == "conf") {
            // this is not enabled yet
        } else {
            if (is_dir($this->workDir)) {
                exec(escapeshellcmd('rm -rf '.$this->workDir));
            }
            mkdir($this->workDir, 0700);
            exec(escapeshellcmd('tar -xzvf '.$file->getPathname().' -C '.$this->workDir),$result);
            // don't check version here, just install
            $result = $this->_installFiles();
            if ($result == 1) {
                // all fine, set CloudMatic data in DB
                $lines = file($this->workDir.'/mhcfg');
                foreach ($lines as $line) {
                    if (stristr($line,'userkennung')) {
                        $data = explode('=',$line);
                        $this->hg->setData('CloudMatic','username', trim($data[1]));
                    }
                    if (stristr($line,'userid')) {
                        $data = explode('=',$line);
                        $this->hg->setData('CloudMatic','userid', trim($data[1]));
                    }
                    if (stristr($line,'userkey')) {
                        $data = explode('=',$line);
                        $this->hg->setData('CloudMatic','userkey', trim($data[1]));
                    }
                }
                // set version
                $version = file($this->workDir.'/newver');
                $this->hg->setData('CloudMatic','Version',trim($version[0]));
                // unset transferPending
                $this->hg->setData('CloudMatic','transferPending', false);
                // reset snr if any
                $this->hg->setData('CloudMatic','snr','ManualKeyUpload');
                // create update cronjob; not needed for smartha anymore, we're using a global crontab
                if ($this->utils->checkOss() !== false) {
                    exec('crontab -r');
                    $cronjob = fopen("/tmp/cm_cronjob", "w");
                    fwrite($cronjob,"0 */6 * * * /usr/bin/perl -e 'sleep int rand 600'; /usr/bin/homegear -e rs ../admin-ui/bin/keytransfer.php > /dev/null 2>&1".PHP_EOL);
                    fclose($cronjob);
                    exec('crontab /tmp/cm_cronjob');
                    unlink('/tmp/cm_cronjob');
                }
                // clean workdir
                exec(escapeshellcmd('rm -rf '.$this->workDir));
                // remove uploaded file
                unlink($_FILES['keyfile']['tmp_name']);
                // enable & start OpenVPN
                $id = $this->hg->managementServiceCommand("openvpn@cloudmatic", 'enable');
                $result = $this->utils->getCommandStatus($id);
                $id = $this->hg->managementServiceCommand("openvpn@cloudmatic", 'restart');
                $result = $this->utils->getCommandStatus($id);
                if ($result['exitCode'] == 0) {
                    return 'success';
                }
                return 'Update installed, but OpenVPN could not be restarted:<br>'.$result['exitCode'].' - '.$result['output'];
            }
            return 'Update could not be installed. Please try again later.';
        }
    }

    /**
     * Check if the box is able to register
     * 
     * @return string
     */
    public function checkConnectionForRegistration() {
        $check = file_get_contents($this->baseUrl);
        if ($check === false) { return 'failure'; }
        return 'success';
    }

    /**
     * Check if the mail-address is allready registered
     * 
     * @return bool
     */
    public function checkMail($mail = false) {
        if ($mail === false) { return; }
        $postdata = http_build_query(array('mail' => $mail));
        $opts = array('http' => array('method'  => 'POST', 'header'  => 'Content-type: application/x-www-form-urlencoded', 'content' => $postdata));
        $context  = stream_context_create($opts);
        $check = file_get_contents($this->mailCheck, false, $context);
        return $check;
    }

    /**
     * Register CloudMatic Account
     */
    public function doRegister($mail = false, $fname = false, $lname = false, $company = false, $street = false, $zip = false, $town = false, $country = false, $vat = false) {
        if ($mail === false || $fname === false || $lname === false || $street === false || $zip === false || $town === false || $country === false) {
            return "An error occured: data missing.<br><br>Please check your input and try again.";
        }
        // generate random snr, needed for initial key transfer
        $snr = bin2hex(random_bytes(30));
        // build request
        $postdata = http_build_query(array(
            'MAIL' => $mail,
            'FNAME' => $fname,
            'LNAME' => $lname,
            'FIRMA' => $company,
            'STRASSE' => $street,
            'PLZ' => $zip,
            'ORT' => $town,
            'LAND' => $country,
            'USTID' => $vat,
            'cb_snr' => $snr,
            'cb_tkey' => $snr
        ));
        $opts = array('http' => array('method'  => 'POST', 'header'  => 'Content-type: application/x-www-form-urlencoded', 'content' => $postdata));
        $context  = stream_context_create($opts);
        $check = file_get_contents($this->registerUrl.'?AUTHKEY='.$this->key, false, $context);
        // check if registration was successfull
        //$check = "Status:OK";
        if (stristr($check,'Status:OK')) {
            // successfull registration, store data needed for initial key transfer
            $this->hg->setData('CloudMatic','transferPending',true);
            $this->hg->setData('CloudMatic','snr',$snr);
            // create cronjob for initial key transfer
            $cronjob = fopen("/tmp/$snr", "w");
            if ($this->utils->checkOss() === false) {
                if (file_exists(__DIR__."/../../../bin/keytransfer.hgs")) {
                    fwrite($cronjob,'*/1 * * * * /usr/bin/homegear -e rs ../ui/bin/keytransfer.hgs > /dev/null 2>&1'.PHP_EOL);
                } else {
                    fwrite($cronjob,'*/1 * * * * /usr/bin/homegear -e rs ../ui/bin/keytransfer.php > /dev/null 2>&1'.PHP_EOL);
                }
            } else {
                fwrite($cronjob,'*/1 * * * * /usr/bin/homegear -e rs ../admin-ui/bin/keytransfer.php > /dev/null 2>&1'.PHP_EOL);
            }
            fclose($cronjob);
            exec('crontab /tmp/'.$snr);
            unlink('/tmp/'.$snr);
        }
        return $check;
    }

    /**
     * Remove the CloudMatic account from central
     */
    public function removeAccount()
    {
        // disable & stop vpn-service
        $id = $this->hg->managementServiceCommand("openvpn@cloudmatic", 'disable');
        $result = $this->utils->getCommandStatus($id);
        $id = $this->hg->managementServiceCommand("openvpn@cloudmatic", 'stop');
        $result = $this->utils->getCommandStatus($id);
        // delete cronjob
        exec('crontab -r');
        // remove files
        $this->_removeFiles();
        // remove data from DB
        $this->hg->deleteData('CloudMatic');
    }

    public function enableSupport()
    {
        // generate password for account
        // ALERT: this is not cryptographically secure!
        $pwd = $this->utils->rand_passwd();
        $cm = $this->hg->getData('CloudMatic');
        if (is_array($cm) && isset($cm['userid'])) {
            // build request
            $postdata = http_build_query(array(
                'mode' => 1,
                'id' => $cm['userid'],
                'key' => $cm['userkey'],
                'username' => 'cmsupport',
                'pwd' => $pwd
            ));
            $opts = array('http' => array('method'  => 'POST', 'header'  => 'Content-type: application/x-www-form-urlencoded', 'content' => $postdata));
            $context  = stream_context_create($opts);
            $check = file_get_contents($this->baseUrl.'script/supportMode.php', false, $context);
            if ($check == 1) {
                // everything was fine, so go on!
                // create smartha-admin-user for support
                if ($this->hg->userExists('cmsupport') !== true) {
                    $this->hg->createUser('cmsupport',$pwd,array(1));
                    $this->hg->setUserMetadata('cmsupport',['expert' => true]);
                } else {
                    // user allready exists, update password
                    $this->hg->updateUser('cmsupport',$pwd,array(1));
                    $this->hg->setUserMetadata('cmsupport',['expert' => true]);
                }
                // check if shell-support-user exists
                exec("grep -c '^cmsupport:' /etc/passwd",$user);
                if(isset($user) && !empty($user) && $user[0] == 1) {
                    // user exists, set password
                    if ($this->utils->checkManagementRunning(false)) {
                        try {
                            $this->hg->managementSetUserPassword('cmsupport',$pwd);
                        } catch (\Homegear\HomegearException $e) { }
                    }
                }
                // check if sshd is running
                exec("pgrep sshd", $pid);
                $sshd = true;
                if (!isset($pid) || empty($pid)) {
                    // sshd not running, so start it
                    $sshd = false;
                    if ($this->utils->checkManagementRunning(false)) {
                        try {
                            $id = $this->hg->managementServiceCommand('ssh','start');
                            $result = $this->utils->getCommandStatus($id);
                        } catch (\Homegear\HomegearException $e) { }
                    }
                }
                $this->hg->setData('CloudMatic','support',['active' => true, 'password' => $pwd, 'sshd_was_active' => $sshd]);
            }
        }
    }

    public function disableSupport()
    {
        $data = $this->hg->getData('CloudMatic','support');
        $cm = $this->hg->getData('CloudMatic');
        if (is_array($cm) && isset($cm['userid'])) {
            // build request
            $postdata = http_build_query(array(
                'mode' => 2,
                'id' => $cm['userid'],
                'key' => $cm['userkey'],
                'username' => 'cmsupport'
            ));
            $opts = array('http' => array('method'  => 'POST', 'header'  => 'Content-type: application/x-www-form-urlencoded', 'content' => $postdata));
            $context  = stream_context_create($opts);
            $check = file_get_contents($this->baseUrl.'script/supportMode.php', false, $context);
        }
        // generate random password
        $pwd = bin2hex(random_bytes(100));
        // delete smartha-admin-supportuser
        if ($this->hg->userExists('cmsupport') === true) {
            $this->hg->deleteUser('cmsupport');
        }
        // check if shell-support-user exists
        exec("grep -c '^cmsupport:' /etc/passwd",$user);
        if(isset($user) && !empty($user) && $user[0] == 1) {
            // user exists, set password
            if ($this->utils->checkManagementRunning(false)) {
                try {
                    $this->hg->managementSetUserPassword('cmsupport',$pwd);
                } catch (\Homegear\HomegearException $e) { }
            }
        }
        // stop sshd if it has been started with enableSupport()
        if (isset($data) && is_array($data) && array_key_exists('sshd_was_active', $data) && $data['sshd_was_active'] === false) {
            if ($this->utils->checkManagementRunning(false)) {
                try {
                    $id = $this->hg->managementServiceCommand('ssh','stop');
                    $result = $this->utils->getCommandStatus($id);
                } catch (\Homegear\HomegearException $e) { }
            }
        }
        $this->hg->deleteData('CloudMatic','support');
    }

    public function getOcservConfig()
    {
        $config = [];
        $knx = false;
        // check if ocserv is installed
        if (!$this->hg->managementDpkgPackageInstalled("ocserv")) { return $config; }
        // check if knx module ist installed - currently we only need ocserv for knx!
        // $knx = array_search('mod_knx.so', array_column($this->hg->listModules(), 'FILENAME'));
        // if ($knx === false) { return $config; }
        $config['network'] = $this->hg->managementGetConfigurationEntry("../ocserv/ocserv.conf","ipv4-network");
        $id = $this->hg->managementServiceCommand('ocserv', 'status');
        $result = $this->utils->getCommandStatus($id);
        if (array_key_exists('output',$result) && $result['output'] != '') {
            $config['enabled'] = stristr($result['output'],'enabled;') ? 'true' : 'false';
        }
        return $config;
    }

    public function getOcservUsers()
    {
        if (!$this->utils->checkManagementRunning(false) || $this->ocservUserdb === false) { return; }
        $users_file = explode("\n",$this->hg->smarthaOcservListuser($this->ocservUserdb));
        if (array_key_exists(0,$users_file) && $users_file[0] == '') {
            unset($users_file[0]);
        }
        if (is_array($users_file) && count($users_file) > 0) {
            foreach ($users_file as $user) {
                $users[] = ['name' => explode(':',$user)[0]];
            }
            return $users;
        }
        return [];
    }

    public function setOcservConfig($params = false)
    {
        if ($params === false) { return; }
        // check if ocserv is installed
        if (!$this->hg->managementDpkgPackageInstalled("ocserv")) { return; }
        $this->hg->managementSetConfigurationEntry("../ocserv/ocserv.conf","ipv4-network",$params['network']);
        if (array_key_exists('enabled', $params)) {
            if ($params['enabled'] == 'true') {
                $id = $this->hg->managementServiceCommand('ocserv','enable');
                $result = $this->utils->getCommandStatus($id);        
                $id = $this->hg->managementServiceCommand('ocserv','restart');
                $result = $this->utils->getCommandStatus($id);        
            } else {
                $id = $this->hg->managementServiceCommand('ocserv','disable');
                $result = $this->utils->getCommandStatus($id);        
                $id = $this->hg->managementServiceCommand('ocserv','stop');
                $result = $this->utils->getCommandStatus($id);        
            }
        }
        return ['result' => 'success'];
    }

    public function addOcservUser($params = false)
    {
        if (!$this->utils->checkManagementRunning(false) || $this->ocservUserdb === false || $params === false) { return; }
        $this->hg->smarthaOcservAdduser($params['username'],$params['password'],$this->ocservUserdb);
    }

    public function deleteOcservUser($params = false)
    {
        if (!$this->utils->checkManagementRunning(false) || $this->ocservUserdb === false || $params === false) { return; }
        $this->hg->smarthaOcservRemoveuser($params['username'],$this->ocservUserdb);
    }

    /**
     * Load data for the current CloudMatic subscription
     * 
     * @return array()
     */
    private function _getSubscriptionData($cmid = false, $cmkey = false) {
        if ($cmid === false || $cmkey === false) { return; }
        $result = file_get_contents($this->baseUrl.'checksmsguthaben.php?id='.$cmid.'&key='.$cmkey);
        if ($result != '') {
            $result = explode("\n",$result);
            foreach ($result as $line) {
                if (stristr($line,'Premium')) {
                    $data['sms'] = explode('=',$line)[1];
                }
                if (stristr($line,'Typ')) {
                    $data['subscription'] = explode('=',$line)[1];
                }
                if (stristr($line,'Key')) {
                    $data['valid_until'] = explode('=',$line)[1];
                }
            }
            if (array_key_exists('valid_until', $data) && stristr($data['valid_until'],'Fehler')) {
                $data['subscription'] = '';
            } else {
                $data['subscription'] = 'CloudMatic connect';
            }
            return (isset($data) ? $data : '');
        }
        return;
    }

    /**
     * Get the current PTP-Address
     * 
     * @return string
     */
    private function _getPTPAddress() {
        $tmp = shell_exec('ifconfig');
        $tmp = preg_split("/\n\n/",$tmp);
        foreach ($tmp as $k => $v) {
            if ($v == '') { continue; }
            // TODO: this is not reliable!
            $interface = trim(substr($v, 0, strpos($v, ': flags')));
            if ($interface == '') {
                $interface = trim(substr($v, 0, strpos($v, 'Link')));
            }
            if (strpos($interface,'lo') === false) {
                $ip = array();
                // TODO: this is not reliable, too!
                if (stristr($v,'P-t-P')) {
                    preg_match('/P\-t\-P\:\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip);
                } elseif (stristr($v,'destination')) {
                    preg_match('/destination \d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip);
                }
                if (array_key_exists(0,$ip)) {
                    if (stristr(':',$ip[0])) {
                        $ip = explode(':',$ip[0]);
                        return $ip[1];
                    } else {
                        $ip = explode(' ',$ip[0]);
                        return $ip[1];
                    }
                }
            }
        }
    }

    /**
     * Download the keyfiles
     * 
     * @return
     */
    private function _keyDownload($url = false) {
        if ($url === false) { return; }
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $data = curl_exec($ch);
        $file = fopen($this->workDir.'/key.tar.gz','w');
        fputs($file,$data);
        fclose($file);
    }

    /**
     * Install the keyfile
     * 
     * @return bool
     */
    private function _installFiles() {
        $res = 0;
        // check if needed files exist
        if (file_exists($this->workDir.'/client.conf') && file_exists($this->workDir.'/client.crt') && file_exists($this->workDir.'/client.key') && file_exists($this->workDir.'/mhca.crt') && file_exists($this->workDir.'/mhcfg')) {
            // install needed files
            foreach (['client.conf', 'client.crt', 'client.key', 'mhca.crt', 'mhcfg'] as $file) {
                $data[$file] = file_get_contents($this->workDir.'/'.$file);
            }
            // adjust paths
            $data['client.conf'] = preg_replace('/\/usr\/local\/etc\/config\/addons\/mh/','/etc/openvpn/cloudmatic',$data['client.conf']);
            try {
                $this->hg->managementWriteCloudMaticConfig($data['client.conf'],$data['mhcfg'],$data['mhca.crt'],$data['client.crt'],$data['client.key']);
            } catch(\Homegear\HomegearException $e) {
                die('Exception catched. Code: '.$e->getCode().'. Message: '.$e->getMessage());
            }
            $res = 1;
        }
        return $res;
    }

    /**
     * Remove the keyfiles
     * 
     * @return none
     */
    private function _removeFiles()
    {
        // we're not able to delete the files (yet), so overwrite them with empty ones
        foreach (['client.conf', 'client.crt', 'client.key', 'mhca.crt', 'mhcfg'] as $file) {
            $data[$file] = '';
        }
        try {
            $this->hg->managementWriteCloudMaticConfig($data['client.conf'],$data['mhcfg'],$data['mhca.crt'],$data['client.crt'],$data['client.key']);
        } catch(\Homegear\HomegearException $e) {
            die('Exception catched. Code: '.$e->getCode().'. Message: '.$e->getMessage());
        }
    }
}