<?php
Namespace App\Model\Admin;

use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use App\Model\HomegearRole as BaseRole;

/**
 * Provides admin functions for Homegear group management
 *
 */
class HomegearRole extends BaseRole
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Load all roles
     * 
     * @return array
     */
    public function loadAll()
    {
        return $this->hg->getRoles();
    }

    /**
     * Load a role by id
     * 
     * @return array
     */
    public function loadById($id = null)
    {
        $allRoles = $this->loadAll();
        if ($id === null) { 
            // no id given, try to load first room
            if (is_array($allRoles) && count($allRoles) > 0) {
                $id = $allRoles[0]['ID'];
            }
        }
        if ($id !== null) {
            // get the role metadata out of the role-array
            $result = $allRoles[array_search($id, array_column($allRoles,'ID'))];
            // load devices
            $variables = $this->hg->getVariablesInRole(intval($id));
            foreach ($variables as $device => $value) {
                foreach ($value as $channel => $data) {
                    if ($channel == -1) { continue; }
                    foreach ($data as $key => $variable) {
                        $result['devices'][] = ['ID' => $device, 'channel' => $channel, 'variable' => $key];
                    }
                }
            }
            if (array_key_exists('devices',$result)) {
                foreach ($result['devices'] as $k => $v) {
                    $key = 0;
                    if (isset($sort) && is_array($sort) && count($sort) > 0) {
                        $key = array_search($v['ID']."-".$v['channel']."-".$v['variable'], $sort);
                    }
                    $v['sort'] = (isset($key) ? $key : 0);
                    $result['devices'][$k] = $v;
                }
                $result['devices'] = ((is_array($result['devices']) && count($result['devices']) > 0) ? array_values($result['devices']) : null);
            }
            return $result;
        }
    }

    /**
     * Update a role name
     * 
     * @return none
     */
    public function update($id = null, $name = null)
    {
        if ($id === null || $name === null) { return; }
        if (!is_array($name)) {
            $name = [$this->defaultLanguage => "$name"];
        }
        foreach ($name as $l => $n) {
            if ($name == '') {
                unset($name[$l]);
            }
        }
        // Update name
        try {
            $this->hg->updateRole(intval($id),$name);
        } catch(\Homegear\HomegearException $e) {
            //return $e;
        }
    }

    /**
     * Delete a role
     * 
     * @return none
     */
    public function delete($id = null) {
        if ($id === null) { return; }
        try {
            $this->hg->deleteRole(intval($id));
        } catch(\Homegear\HomegearException $e) {
            //return $e;
        }
    }

    /**
     * Add a variable to a role
     * 
     * @return none
     */
    public function addDevice($id = null, $deviceId = null, $channelId = null, $variable = null)
    {
        if ($id === null || $deviceId === null || $channelId === null || $variable === null) { return; }
        try {
            $this->hg->addRoleToVariable(intval($deviceId), intval($channelId), "$variable", intval($id));
        } catch(\Homegear\HomegearException $e) {
            //return $e;
        }
    }

    /**
     * Remove a variable from a role
     * 
     * @return none
     */
    public function removeDevice($id = null, $deviceId = null, $channelId = null, $variable = null)
    {
        if ($id === null || $deviceId === null || $channelId === null || $variable === null) { return; }
        try {
            $this->hg->removeRoleFromVariable(intval($deviceId), intval($channelId), "$variable", intval($id));
        } catch(\Homegear\HomegearException $e) {
            //return $e;
        }
    }
}